/* $Id: dlist.h,v 0.2 1998/05/03 21:42:06 rsmit06 Stab $
 *
 * Public header file for double linked list library
 *
 * Copyright (C) 1997, 1998  R.F. Smith
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: dlist.h,v $
 * Revision 0.2  1998/05/03 21:42:06  rsmit06
 * Added functions, changed several prototypes.
 *
 * Revision 0.1  1997/12/21 16:52:47  rsmit06
 * Initial code
 *
 */

#ifndef _DOUBLE_LIST_H
#define _DOUBLE_LIST_H

/* Don't access these structures directly. Use the functions. */

struct _dlink {
  struct _dlink *prev;          /* pointer to previous link */
  struct _dlink *next;          /* pointer to next link */
  void *pdata;                  /* pointer to data */
  unsigned cb_xtra;             /* extra bytes allocated to hold data */
  int type;                     /* to identify a link by type */
};

struct _dlist {
  struct _dlink *head;         /* head of the list */
  struct _dlink *tail;         /* tail of the list */
  struct _dlink *current;      /* current link */
  unsigned num_items;          /* number of items in the list */
};

/*typedef struct _dlink *dlink_p;*/
typedef struct _dlist *dlist_p;

/* Creates a linked list */
extern dlist_p dl_create(void);
/* Destroys a linked list. All allocated data will be destroyed */
extern void dl_destroy(dlist_p list);
/* Erases the list */
extern void dl_erase(dlist_p list);

/* Prepends a link to the head of the list. */
extern void dl_prepend(dlist_p list, unsigned cb, void *pdata, int type);
/* Appends a link to the tail of the list. */
extern void dl_append(dlist_p list, unsigned cb, void *pdata, int type);

/* Get the properties of the current item. */
extern void *dl_cur_data(dlist_p list);
extern int dl_cur_type(dlist_p list);
extern unsigned dl_cur_size(dlist_p list);
/* Gets the first link from a list */
extern void *dl_first(dlist_p list);
/* Gets the last link from a list */
extern void *dl_last(dlist_p list);
/* Gets the next link from a link */
extern void *dl_next(dlist_p link);
/* Gets the previous link from a link */
extern void *dl_prev(dlist_p link);

/* Remove the first link from the list */
extern void dl_rm_head(dlist_p list);
/* Remove the last link from the list */
extern void dl_rm_tail(dlist_p list);
/* Remove the current link from the list. */
extern void dl_rm(dlist_p list);

/* Finds a link whose data is identical to `target', and who's `type'
   matches the given value. */
/*extern void *dl_find(dlist_p list, int (*compare)(void*,void*),
                     void *target, int type);*/

#endif /* _DOUBLE_LIST_H */
